using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.ComponentModel;
using MySql.Data.VisualStudio.Editors;
using MySql.Data.VisualStudio.DbObjects;
using System.Windows.Forms.Design;

namespace MySql.Data.VisualStudio.DbObjects
{
    internal class Column : Object, ITablePart
	{
        internal struct ColumnProperties
        {
            private string characterSet;

            public ColumnProperties(DataRow row)
                : this()
            {
                AllowNull = true;
                if (row != null)
                {
                    ColumnName = row["COLUMN_NAME"].ToString();
                    AllowNull = row["IS_NULLABLE"] != DBNull.Value && row["IS_NULLABLE"].ToString() == "YES";
                    Comment = row["COLUMN_COMMENT"].ToString();
                    Collation = row["COLLATION_NAME"].ToString();
                    CharacterSet = row["CHARACTER_SET_NAME"].ToString();
                    DefaultValue = row["COLUMN_DEFAULT"].ToString();

                    string columnType = row["COLUMN_TYPE"].ToString().ToLowerInvariant();
                    int index = columnType.IndexOf(' ');
                    if (index == -1)
                        index = columnType.Length;
                    DataType = columnType.Substring(0, index);
                    CleanDataType();

                    columnType = columnType.Substring(index);
                    IsUnsigned = columnType.IndexOf("unsigned") != -1;
                    IsZerofill = columnType.IndexOf("zerofill") != -1;

                    PrimaryKey = row["COLUMN_KEY"].ToString() == "PRI";
                    Precision = DataRowHelpers.GetValueAsInt32(row, "NUMERIC_PRECISION");
                    Scale = DataRowHelpers.GetValueAsInt32(row, "NUMERIC_SCALE");

                    string extra = row["EXTRA"].ToString().ToLowerInvariant();
                    if (extra != null)
                        AutoIncrement = extra.IndexOf("auto_increment") != -1;
                }               
            }

            [Category("General")]
            [Description("The name of this column")]
            public string ColumnName { get; set; }

            [Category("General")]
            [DisplayName("Data Type")]
            [TypeConverter(typeof(DataTypeConverter))]
            [RefreshProperties(RefreshProperties.All)]
            public string DataType { get; set; }

            [TypeConverter(typeof(YesNoTypeConverter))]
            [Category("Options")]
            [DisplayName("Allow Nulls")]
            public bool AllowNull { get; set; }

            [TypeConverter(typeof(YesNoTypeConverter))]
            [Category("Options")]
            [DisplayName("Is Unsigned")]
            public bool IsUnsigned { get; set; }

            [TypeConverter(typeof(YesNoTypeConverter))]
            [Category("Options")]
            [DisplayName("Is Zerofill")]
            public bool IsZerofill { get; set; }

            [Category("General")]
            [DisplayName("Default Value")]
            public string DefaultValue { get; set; }

            [TypeConverter(typeof(YesNoTypeConverter))]
            [Category("Options")]
            [DisplayName("Autoincrement")]
            public bool AutoIncrement { get; set; }

            [TypeConverter(typeof(YesNoTypeConverter))]
            [Category("Options")]
            [DisplayName("Primary Key")]
            [RefreshProperties(RefreshProperties.All)]
            public bool PrimaryKey { get; set; }

            public int Precision { get; set; }
            public int Scale { get; set; }


            [Category("Encoding")]
            [DisplayName("Character Set")]
            [TypeConverter(typeof(CharacterSetTypeConverter))]
            public string CharacterSet
            {
                get { return characterSet; }
                set
                {
                    if (value != characterSet)
                        Collation = String.Empty;
                    characterSet = value;
                }
            }

            [Category("Encoding")]
            [TypeConverter(typeof(CollationTypeConverter))]
            public string Collation { get; set; }

            [Category("Miscellaneous")]
            public string Comment { get; set; }

            public void CleanDataType()
            {
                if (DataType.Contains("char") || DataType.Contains("binary")) return;
                int index = DataType.IndexOf("(");
                if (index == -1) return;
                DataType = DataType.Substring(0, index);
            }

            #region Methods needed so PropertyGrid won't bold our values

            private bool ShouldSerializeColumnName() { return false; }
            private bool ShouldSerializeDataType() { return false; }
            private bool ShouldSerializeAllowNull() { return false; }
            private bool ShouldSerializeIsUnsigned() { return false; }
            private bool ShouldSerializeIsZerofill() { return false; }
            private bool ShouldSerializeDefaultValue() { return false; }
            private bool ShouldSerializeAutoIncrement() { return false; }
            private bool ShouldSerializePrimaryKey() { return false; }
            private bool ShouldSerializePrecision() { return false; }
            private bool ShouldSerializeScale() { return false; }
            private bool ShouldSerializeCharacterSet() { return false; }
            private bool ShouldSerializeCollation() { return false; }
            private bool ShouldSerializeComment() { return false; }

            #endregion
        }

        ColumnProperties current = new ColumnProperties();
        ColumnProperties old = new ColumnProperties();
        private Table owningTable;
        private bool isNew = true;

		public Column(DataRow row)
		{
            if (row != null)
            {
                old = current = new ColumnProperties(row);
                isNew = false;
            }
        }

        #region Mirrored Properties 

        public string ColumnName
        {
            get { return current.ColumnName; }
            set { current.ColumnName = value; }
        }

        public string CharacterSet 
        { 
            get { return current.CharacterSet; } 
        }

        public bool PrimaryKey
        {
            get { return current.PrimaryKey; }
            set { current.PrimaryKey = value; }
        }

        public string DataType
        {
            get { return current.DataType; }
            set { current.DataType = value; }
        }

        #endregion

        public ColumnProperties Properties
        {
            get { return current; }
        }

        internal Table OwningTable
        {
            get { return owningTable; }
            set { owningTable = value; }
        }

        #region ITablePart Members

        bool ITablePart.IsNew()
        {
            return isNew;
        }

        string ITablePart.GetDropSql()
        {
            return String.Format("DROP `{0}`", old.ColumnName);
        }

        string ITablePart.GetSql(bool newTable)
        {
            if (current.Equals(old)) return null;

            if (String.IsNullOrEmpty(ColumnName)) return null;

            StringBuilder sql = new StringBuilder();
            if (!newTable)
            {
                if (isNew)
                    sql.Append("ADD ");
                else
                    sql.AppendFormat("CHANGE `{0}`", old.ColumnName);
            }

            sql.AppendFormat(" `{0}` {1}", ColumnName, DataType);
            if (current.CharacterSet != old.CharacterSet)
                sql.AppendFormat(" CHARACTER SET '{0}'", current.CharacterSet);
            if (current.Collation != old.Collation)
                sql.AppendFormat(" COLLATE '{0}'", current.Collation);
            if (current.AllowNull != old.AllowNull)
                sql.Append(current.AllowNull ? " NULL" : " NOT NULL");
            if (current.DefaultValue != old.DefaultValue)
                sql.AppendFormat(" DEFAULT '{0}'", current.DefaultValue);
            if (current.Comment != old.Comment)
                sql.AppendFormat(" COMMENT '{0}'", current.Comment);
            if (current.IsUnsigned != old.IsUnsigned) 
                sql.Append(" UNSIGNED");
            if (current.IsZerofill != old.IsZerofill) 
                sql.Append(" ZEROFILL");
            if (current.AutoIncrement != old.AutoIncrement) 
                sql.Append(" AUTO_INCREMENT");

            return sql.ToString();
        }

        #endregion
    }
}
